@@ -1,6 +1,6 @@
 ;;; tramp-cache.el --- file information caching for Tramp  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2000, 2005-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2000, 2005-2025 Free Software Foundation, Inc.
 
 ;; Author: Daniel Pittman <daniel@inanna.danann.net>
 ;;         Michael Albinus <michael.albinus@gmx.de>
@@ -68,23 +68,29 @@
 
 ;; Some properties are handled special:
 ;;
-;; - "process-name", "process-buffer", "first-password-request" and
-;;   "pw-spec" are not saved in the file
+;; - Ephemeral properties which start with a space, like
+;;   " process-name", are not saved in the file
 ;;   `tramp-persistency-file-name', although being connection
 ;;   properties related to a `tramp-file-name' structure.
 ;;
+;; - Properties retrieved from `tramp-connection-properties' are not
+;;   saved in the file `tramp-persistency-file-name'.
+;;
 ;; - Reusable properties, which should not be saved, are kept in the
 ;;   process key retrieved by `tramp-get-process' (the main connection
 ;;   process).  Other processes could reuse these properties, avoiding
 ;;   recomputation when a new asynchronous process is created by
 ;;   `make-process'.  Examples are "unsafe-temporary-file",
 ;;   "remote-path", "device" (tramp-adb.el) or "share" (tramp-gvfs.el).
+;;   FIXME: Shall they be converted to ephemeral connection properties?
 
 ;;; Code:
 
 (require 'tramp-compat)
 (require 'time-stamp)
 
+(declare-function tramp-get-method-parameter "tramp")
+
 ;;; -- Cache --
 
 ;;;###tramp-autoload
@@ -97,19 +103,25 @@
 Every entry has the form (REGEXP PROPERTY VALUE).  The regexp
 matches remote file names.  It can be nil.  PROPERTY is a string,
 and VALUE the corresponding value.  They are used, if there is no
-matching entry for PROPERTY in `tramp-cache-data'.  For more
-details see the info pages."
+matching entry for PROPERTY in `tramp-cache-data'.
+
+PROPERTY can also be a string representing a parameter in
+`tramp-methods'.  For more details see the Info node `(tramp) Predefined
+connection information'."
   :group 'tramp
   :version "24.4"
   :type '(repeat (list (choice :tag "File Name regexp" regexp (const nil))
 		       (choice :tag "        Property" string)
-		       (choice :tag "           Value" sexp))))
+		       (choice :tag "           Value" sexp)))
+  :link '(info-link :tag "Tramp manual"
+		    "(tramp) Predefined connection information"))
 
 ;;;###tramp-autoload
 (defcustom tramp-persistency-file-name (locate-user-emacs-file "tramp")
   "File which keeps connection history for Tramp connections."
   :group 'tramp
-  :type 'file)
+  :type 'file
+  :link '(info-link :tag "Tramp manual" "(tramp) Connection caching"))
 
 ;;;###tramp-autoload
 (defconst tramp-cache-version (make-tramp-file-name :method "cache")
@@ -122,23 +134,32 @@ details see the info pages."
 (defconst tramp-cache-undefined 'undef
   "The symbol marking undefined hash keys and values.")
 
+;;;###tramp-autoload
 (defun tramp-get-hash-table (key)
   "Return the hash table for KEY.
 If it doesn't exist yet, it is created and initialized with
 matching entries of `tramp-connection-properties'.
 If KEY is `tramp-cache-undefined', don't create anything, and return nil."
-  (declare (tramp-suppress-trace t))
-  (unless (eq key tramp-cache-undefined)
-    (or (gethash key tramp-cache-data)
-	(let ((hash
-	       (puthash key (make-hash-table :test #'equal) tramp-cache-data)))
-	  (when (tramp-file-name-p key)
-	    (dolist (elt tramp-connection-properties)
-	      (when (string-match-p
-		     (or (nth 0 elt) "")
-		     (tramp-make-tramp-file-name key 'noloc))
-		(tramp-set-connection-property key (nth 1 elt) (nth 2 elt)))))
-	  hash))))
+  ;; (declare (tramp-suppress-trace t))
+  (let ((tramp-verbose 0))
+    (unless (eq key tramp-cache-undefined)
+      (or (gethash key tramp-cache-data)
+	  (let ((hash
+		 (puthash key (make-hash-table :test #'equal) tramp-cache-data)))
+	    (when (tramp-file-name-p key)
+	      (dolist (elt tramp-connection-properties)
+		(when (string-match-p
+		       (or (nth 0 elt) "")
+		       (tramp-make-tramp-file-name key 'noloc))
+		  ;; Mark it as taken from `tramp-connection-properties'.
+		  (tramp-set-connection-property
+		   key (propertize (nth 1 elt) 'tramp-default t) (nth 2 elt)))))
+	    hash)))))
+
+;; We cannot use the `declare' form for `tramp-suppress-trace' in
+;; autoloaded functions, because the tramp-loaddefs.el generation
+;; would fail.
+(function-put #'tramp-get-hash-table 'tramp-suppress-trace t)
 
 ;;;###tramp-autoload
 (defun tramp-get-file-property (key file property &optional default)
@@ -271,8 +292,7 @@ Remove also properties of all files in subdirectories."
     (dolist (key (hash-table-keys tramp-cache-data))
       (when (and (tramp-file-name-p key)
 		 (stringp (tramp-file-name-localname key))
-		 (tramp-compat-string-search
-		  directory (tramp-file-name-localname key)))
+		 (string-search directory (tramp-file-name-localname key)))
 	(remhash key tramp-cache-data)))
     ;; Remove file properties of symlinks.
     (when (and (stringp truename)
@@ -470,8 +490,10 @@ used to cache connection properties of the local machine."
 	  (hash (tramp-get-hash-table key))
 	  (cached (and (hash-table-p hash)
 		       (gethash ,property hash tramp-cache-undefined))))
+     (tramp-message key 7 "Saved %s %s" ,property cached)
      (unwind-protect (progn ,@body)
        ;; Reset PROPERTY.  Recompute hash, it could have been flushed.
+       (tramp-message key 7 "Restored %s %s" ,property cached)
        (setq hash (tramp-get-hash-table key))
        (if (not (eq cached tramp-cache-undefined))
 	   (puthash ,property cached hash)
@@ -488,9 +510,13 @@ PROPERTIES is a list of file properties (strings)."
 	   (mapcar
 	    (lambda (property)
 	      (cons property (gethash property hash tramp-cache-undefined)))
-	    ,properties)))
+	    ,properties))
+	  ;; Avoid superfluous debug buffers during host name completion.
+	  (tramp-verbose (if minibuffer-completing-file-name 0 tramp-verbose)))
+     (tramp-message key 7 "Saved %s" values)
      (unwind-protect (progn ,@body)
        ;; Reset PROPERTIES.  Recompute hash, it could have been flushed.
+       (tramp-message key 7 "Restored %s" values)
        (setq hash (tramp-get-hash-table key))
        (dolist (value values)
 	 (if (not (eq (cdr value) tramp-cache-undefined))
@@ -546,7 +572,7 @@ PROPERTIES is a list of file properties (strings)."
      (lambda (key)
        (and (tramp-file-name-p key)
 	    (null (tramp-file-name-localname key))
-	    (tramp-connection-property-p key "process-buffer")
+	    (tramp-connection-property-p key " connected")
 	    key))
      (hash-table-keys tramp-cache-data))))
 
@@ -564,12 +590,11 @@ PROPERTIES is a list of file properties (strings)."
 	    print-length print-level)
 	;; Remove `tramp-null-hop'.
 	(remhash tramp-null-hop cache)
-	;; Remove temporary data.  If there is the key "login-as", we
-	;; don't save either, because all other properties might
-	;; depend on the login name, and we want to give the
-	;; possibility to use another login name later on.  Key
-	;; "started" exists for the "ftp" method only, which must not
-	;; be kept persistent.
+	;; If there is the key "login-as", we don't save, because all
+	;; other properties might depend on the login name, and we
+	;; want to give the possibility to use another login name
+	;; later on.  Key "started" exists for the "ftp" method only,
+	;; which must not be kept persistent.
 	(maphash
 	 (lambda (key value)
 	   (if (and (tramp-file-name-p key) (hash-table-p value)
@@ -579,17 +604,19 @@ PROPERTIES is a list of file properties (strings)."
 		    (not (gethash "login-as" value))
 		    (not (gethash "started" value)))
 	       (progn
-		 (remhash "process-name" value)
-		 (remhash "process-buffer" value)
-		 (remhash "first-password-request" value)
-		 (remhash "pw-spec" value))
+		 (dolist (k (hash-table-keys value))
+		   ;; Suppress ephemeral properties.
+		   (when (or (string-prefix-p " " k)
+			     (get-text-property 0 'tramp-default k))
+		     (remhash k value)))
+		 (unless (hash-table-keys value)
+		   (remhash key cache)))
 	     (remhash key cache)))
 	 cache)
 	;; Dump it.
 	(with-temp-file tramp-persistency-file-name
 	  (insert
-	   ;; Starting with Emacs 28, we could use `lisp-data'.
-	   (format ";; -*- emacs-lisp -*- <%s %s>\n"
+	   (format ";; -*- lisp-data -*- <%s %s>\n"
 		   (time-stamp-string "%02y/%02m/%02d %02H:%02M:%02S")
 		   tramp-persistency-file-name)
 	   ";; Tramp connection history.  Don't change this file.\n"
@@ -612,22 +639,25 @@ example if the host configuration changes often, or if you plug
 your laptop to different networks frequently."
   :group 'tramp
   :version "29.1"
-  :type 'boolean)
+  :type 'boolean
+  :link '(tramp-info-link :tag "Tramp manual" tramp-completion-use-cache))
 
 ;;;###tramp-autoload
 (defun tramp-parse-connection-properties (method)
   "Return a list of (user host) tuples allowed to access for METHOD.
 This function is added always in `tramp-get-completion-function'
 for all methods.  Resulting data are derived from connection history."
-  (and tramp-completion-use-cache
-       (mapcar
-	(lambda (key)
-	  (and (tramp-file-name-p key)
-	       (string-equal method (tramp-file-name-method key))
-	       (not (tramp-file-name-localname key))
-	       (list (tramp-file-name-user key)
-		     (tramp-file-name-host key))))
-	(hash-table-keys tramp-cache-data))))
+  (mapcar
+   (lambda (key)
+     (let ((tramp-verbose 0))
+       (and (tramp-file-name-p key)
+	    (string-equal method (tramp-file-name-method key))
+	    (not (tramp-file-name-localname key))
+	    (tramp-get-method-parameter
+	     key 'tramp-completion-use-cache tramp-completion-use-cache)
+	    (list (tramp-file-name-user key)
+		  (tramp-file-name-host key)))))
+   (hash-table-keys tramp-cache-data)))
 
 ;; When "emacs -Q" has been called, both variables are nil.  We do not
 ;; load the persistency file then, in order to have a clean test environment.
@@ -683,5 +713,7 @@ for all methods.  Resulting data are derived from connection history."
 ;;; TODO:
 ;;
 ;; * Use multisession.el, starting with Emacs 29.1.
+;;
+;; Use `with-memoization', starting with Emacs 29.1.
 
 ;;; tramp-cache.el ends here
