@@ -32,6 +32,7 @@
 
 ;; Pacify byte-compiler.
 (declare-function dired-advertise "dired")
+(declare-function dired-get-file-for-visit "dired")
 (declare-function dired-unadvertise "dired")
 (declare-function mml-mode "mml")
 (declare-function mml-insert-empty-tag "mml")
@@ -56,6 +57,36 @@ SYNTAX can be one of the symbols `default' (default),
   (when syntax
     (customize-set-variable 'tramp-syntax syntax)))
 
+;;;###tramp-autoload
+(defun tramp-enable-method (method)
+  "Enable optional METHOD if possible."
+  (interactive
+   (list
+    (completing-read
+     "method: "
+     (tramp-compat-seq-keep
+      (lambda (x)
+	(when-let* ((name (symbol-name x))
+		    ;; It must match `tramp-enable-METHOD-method'.
+		    ((string-match
+		      (rx "tramp-enable-"
+			  (group (regexp tramp-method-regexp))
+			  "-method")
+		      name))
+		    (method (match-string 1 name))
+		    ;; It must not be enabled yet.
+		    ((not (assoc method tramp-methods))))
+	  method))
+      ;; All method enabling functions.
+      (mapcar
+       #'intern (all-completions "tramp-enable-" obarray #'functionp))))))
+
+  (when-let* (((not (assoc method tramp-methods)))
+	      (fn (intern (format "tramp-enable-%s-method" method)))
+	      ((functionp fn)))
+    (funcall fn)
+    (message "Tramp method \"%s\" enabled" method)))
+
 ;; Use `match-buffers' starting with Emacs 29.1.
 ;;;###tramp-autoload
 (defun tramp-list-tramp-buffers ()
@@ -71,12 +102,35 @@ SYNTAX can be one of the symbols `default' (default),
 ;; Use `match-buffers' starting with Emacs 29.1.
 ;;;###tramp-autoload
 (defun tramp-list-remote-buffers ()
-  "Return a list of all buffers with remote `default-directory'."
+  "Return a list of remote buffers, excluding internal Tramp buffers.
+A buffer is considered remote if either its `default-directory' or
+`buffer-file-name' is a remote file name."
   (tramp-compat-seq-keep
-   (lambda (x)
-     (when (tramp-tramp-file-p (tramp-get-default-directory x)) x))
+   (lambda (buffer)
+     (when (tramp-tramp-file-p
+            (or (buffer-file-name buffer)
+                (tramp-get-default-directory buffer)))
+       buffer))
    (buffer-list)))
 
+;;;###tramp-autoload
+(defun tramp-list-remote-buffer-connections ()
+  "Return a list of all remote buffer connections.
+A buffer is considered remote if either its `default-directory' or
+`buffer-file-name' is a remote file name."
+  (seq-uniq
+   (mapcar (lambda (buffer)
+             (or
+              (when (buffer-file-name buffer)
+                (file-remote-p (buffer-file-name buffer)))
+              (when (tramp-get-default-directory buffer)
+                (file-remote-p (tramp-get-default-directory buffer)))))
+           ;; Eliminate false positives from internal Tramp buffers.
+           (seq-remove
+            (lambda (buffer)
+              (member (buffer-name buffer) (tramp-list-tramp-buffers)))
+            (tramp-list-remote-buffers)))))
+
 ;;; Cleanup
 
 ;;;###tramp-autoload
@@ -88,11 +142,12 @@ Each function is called with the current vector as argument.")
 (defun tramp-cleanup-connection
     (vec &optional keep-debug keep-password keep-processes)
   "Flush all connection related objects.
-This includes password cache, file cache, connection cache,
-buffers, processes.  KEEP-DEBUG non-nil preserves the debug
-buffer.  KEEP-PASSWORD non-nil preserves the password cache.
-KEEP-PROCESSES non-nil preserves the asynchronous processes.
-When called interactively, a Tramp connection has to be selected."
+This includes password cache, file cache, connection cache, buffers,
+processes.  KEEP-DEBUG non-nil preserves the debug and trace buffer.
+KEEP-PASSWORD non-nil preserves the password cache.  KEEP-PROCESSES
+non-nil preserves the asynchronous processes.  When called
+interactively, a Tramp connection has to be selected."
+  (declare (completion tramp-active-command-completion-p))
   (interactive
    ;; When interactive, select the Tramp remote identification.
    ;; Return nil when there is no Tramp connection.
@@ -142,7 +197,7 @@ When called interactively, a Tramp connection has to be selected."
 		     (get-buffer (tramp-debug-buffer-name vec)))
 		   (unless keep-debug
 		     (get-buffer (tramp-trace-buffer-name vec)))
-		   (tramp-get-connection-property vec "process-buffer")))
+		   (tramp-get-connection-property vec " connected")))
       (when (bufferp buf) (kill-buffer buf)))
 
     ;; Flush file cache.
@@ -157,18 +212,12 @@ When called interactively, a Tramp connection has to be selected."
 ;;;###tramp-autoload
 (defun tramp-cleanup-this-connection ()
   "Flush all connection related objects of the current buffer's connection."
-  ;; (declare (completion tramp-command-completion-p)))
+  (declare (completion tramp-command-completion-p))
   (interactive)
   (and (tramp-tramp-file-p default-directory)
        (tramp-cleanup-connection
 	(tramp-dissect-file-name default-directory 'noexpand))))
 
-;; Starting with Emacs 28.1, this can be replaced by the "(declare ...)" form.
-;;;###tramp-autoload
-(function-put
- #'tramp-cleanup-this-connection 'completion-predicate
- #'tramp-command-completion-p)
-
 ;;;###tramp-autoload
 (defvar tramp-cleanup-all-connections-hook nil
   "List of functions to be called after all Tramp connections are cleaned up.")
@@ -177,6 +226,7 @@ When called interactively, a Tramp connection has to be selected."
 (defun tramp-cleanup-all-connections ()
   "Flush all Tramp internal objects.
 This includes password cache, file cache, connection cache, buffers."
+  (declare (completion tramp-active-command-completion-p))
   (interactive)
 
   ;; Flush password cache.
@@ -220,7 +270,8 @@ happens when at least one of the functions returns non-nil.  The
 functions are called with `current-buffer' set."
   :group 'tramp
   :version "30.1"
-  :type 'hook)
+  :type 'hook
+  :link '(info-link :tag "Tramp manual" "(tramp) Cleanup remote connections"))
 
 (add-hook 'tramp-cleanup-some-buffers-hook
 	  #'buffer-file-name)
@@ -273,6 +324,7 @@ functions are called with `current-buffer' set."
 A buffer is killed when it has a remote `default-directory', and
 one of the functions in `tramp-cleanup-some-buffers-hook' returns
 non-nil."
+  (declare (completion tramp-active-command-completion-p))
   (interactive)
 
   ;; Remove all Tramp related connections.
@@ -288,10 +340,28 @@ non-nil."
 ;;;###tramp-autoload
 (defun tramp-cleanup-all-buffers ()
   "Kill all remote buffers."
+  (declare (completion tramp-active-command-completion-p))
   (interactive)
-  (let ((tramp-cleanup-some-buffers-hook '(tramp-compat-always)))
+  (let ((tramp-cleanup-some-buffers-hook '(always)))
     (tramp-cleanup-some-buffers)))
 
+;;;###tramp-autoload
+(defun tramp-cleanup-bufferless-connections ()
+  "Flush connection-related objects for which no buffer exists.
+A bufferless connection is one for which no live buffer's
+`buffer-file-name' or `default-directory' is associated with that
+connection, except for Tramp internal buffers.
+Display a message of cleaned-up connections."
+  (interactive)
+  (when-let* ((bufferless-connections
+               (seq-difference
+                (mapcar #'tramp-make-tramp-file-name (tramp-list-connections))
+                (tramp-list-remote-buffer-connections))))
+    (message "Cleaning up %s" (string-join bufferless-connections ", "))
+    (dolist (connection bufferless-connections)
+      (tramp-cleanup-connection
+       (tramp-dissect-file-name connection 'noexpand)))))
+
 ;;; Rename
 
 (defcustom tramp-default-rename-alist nil
@@ -314,13 +384,15 @@ expression which always matches."
   :group 'tramp
   :version "27.1"
   :type '(repeat (cons (choice :tag "Source regexp" regexp sexp)
-		       (choice :tag "Target   name" string (const nil)))))
+		       (choice :tag "Target   name" string (const nil))))
+  :link '(info-link :tag "Tramp manual" "(tramp) Renaming remote files"))
 
 (defcustom tramp-confirm-rename-file-names t
   "Whether renaming a buffer file name must be confirmed."
   :group 'tramp
   :version "27.1"
-  :type 'boolean)
+  :type 'boolean
+  :link '(info-link :tag "Tramp manual" "(tramp) Renaming remote files"))
 
 (defun tramp-default-rename-file (string)
   "Determine default file name for renaming according to STRING.
@@ -337,7 +409,7 @@ function returns nil"
 	(when (string-match-p (or (eval (car item) t) "") string)
 	  (setq tdra nil
 		result
-		(format-spec
+		(tramp-format-spec
 		 (cdr item) (format-spec-make ?m method ?u user ?h host)))))
       result)))
 
@@ -384,6 +456,7 @@ without confirmation if the prefix argument is non-nil.
 
 The remote connection identified by SOURCE is flushed by
 `tramp-cleanup-connection'."
+  (declare (completion tramp-active-command-completion-p))
   (interactive
    (let ((connections
 	  (mapcar #'tramp-make-tramp-file-name (tramp-list-connections)))
@@ -474,11 +547,11 @@ ESC or `q' to quit without changing further buffers,
 	(dolist (buffer (tramp-list-remote-buffers))
           (switch-to-buffer buffer)
 	  (let* ((bfn (buffer-file-name))
-		 (new-bfn (and (stringp bfn)
-			       (tramp-compat-string-replace source target bfn)))
+		 (new-bfn (and (stringp bfn) (string-replace source target bfn)))
 		 (prompt (format-message
 			  "Set visited file name to `%s' [Type yn!eq or %s] "
-			  new-bfn (key-description (vector help-char)))))
+                          new-bfn (if (fboundp 'help-key) (help-key) ; 29.1
+                                    (key-description (vector help-char))))))
 	    (when (and (buffer-live-p buffer) (stringp bfn)
 		       (string-prefix-p source bfn)
 		       ;; Skip, and don't ask again.
@@ -522,7 +595,7 @@ Interactively, TARGET is selected from `tramp-default-rename-alist'
 without confirmation if the prefix argument is non-nil.
 
 For details, see `tramp-rename-files'."
-  ;; (declare (completion tramp-command-completion-p))
+  (declare (completion tramp-command-completion-p))
   (interactive
    (let ((source default-directory)
 	 target
@@ -553,11 +626,6 @@ For details, see `tramp-rename-files'."
 
   (tramp-rename-files default-directory target))
 
-;; Starting with Emacs 28.1, this can be replaced by the "(declare ...)" form.
-;;;###tramp-autoload
-(function-put
- #'tramp-rename-these-files 'completion-predicate #'tramp-command-completion-p)
-
 ;;; Run as sudo
 
 (defcustom tramp-file-name-with-method "sudo"
@@ -568,7 +636,21 @@ For details, see `tramp-rename-files'."
 		 (const "sudo")
 		 (const "doas")
 		 (const "run0")
-		 (const "ksu")))
+		 (const "ksu"))
+  :link '(tramp-info-link :tag "Tramp manual" tramp-file-name-with-method))
+
+(defmacro with-tramp-file-name-with-method (&rest body)
+  "Ask user for `tramp-file-name-with-method' if needed.
+Run BODY."
+  (declare (indent 0) (debug t))
+  `(let ((tramp-file-name-with-method
+          (if current-prefix-arg
+	      (completing-read
+	       "Tramp method: "
+               (mapcar #'cadr (cdr (get 'tramp-file-name-with-method 'custom-type)))
+               nil t tramp-file-name-with-method)
+            tramp-file-name-with-method)))
+     ,@body))
 
 (defun tramp-file-name-with-sudo (filename)
   "Convert FILENAME into a multi-hop file name with \"sudo\".
@@ -602,35 +684,76 @@ An alternative method could be chosen with `tramp-file-name-with-method'."
      (make-tramp-file-name
       :method tramp-file-name-with-method :localname filename))))
 
-;;;###tramp-autoload
+;; FIXME: We would like to rename this for Emacs 31.1 to a name that
+;; does not encode the default method.  It is intended as a generic
+;; privilege-elevation command.  Some ideas from bug#76974:
+;; `tramp-revert-buffer-obtain-root',
+;; `tramp-revert-buffer-as-superuser'.
+
+;;;###autoload
 (defun tramp-revert-buffer-with-sudo ()
-  "Revert current buffer to visit with \"sudo\" permissions.
-An alternative method could be chosen with `tramp-file-name-with-method'.
+  "Visit the current file again with superuser, or root, permissions.
+
+By default this is done using the \"sudo\" Tramp method.
+You can customize `tramp-file-name-with-method' to change this.
+
+Interactively, with a prefix argument, prompt for a different method.
+
 If the buffer visits a file, the file is replaced.
 If the buffer runs `dired', the buffer is reverted."
   (interactive)
-  (cond
-   ((buffer-file-name)
-    (let ((pos (point)))
-      (find-alternate-file (tramp-file-name-with-sudo (buffer-file-name)))
-      (goto-char pos)))
-   ((tramp-dired-buffer-p)
-    (dired-unadvertise (expand-file-name default-directory))
-    (setq default-directory (tramp-file-name-with-sudo default-directory)
-	  list-buffers-directory
-	  (tramp-file-name-with-sudo list-buffers-directory))
-    (if (consp dired-directory)
-	(setcar
-	 dired-directory (tramp-file-name-with-sudo (car dired-directory)))
-      (setq dired-directory (tramp-file-name-with-sudo dired-directory)))
-    (dired-advertise)
-    (revert-buffer))))
+  (with-tramp-file-name-with-method
+    (cond
+     ((buffer-file-name)
+      (let ((pos (point)))
+        (find-alternate-file (tramp-file-name-with-sudo (buffer-file-name)))
+        (goto-char pos)))
+     ((tramp-dired-buffer-p)
+      (dired-unadvertise (expand-file-name default-directory))
+      (setq default-directory (tramp-file-name-with-sudo default-directory)
+	    list-buffers-directory
+	    (tramp-file-name-with-sudo list-buffers-directory))
+      (if (consp dired-directory)
+	  (setcar
+	   dired-directory (tramp-file-name-with-sudo (car dired-directory)))
+        (setq dired-directory (tramp-file-name-with-sudo dired-directory)))
+      (dired-advertise)
+      (revert-buffer)))))
+
+;; This function takes action, when `read-extended-command-predicate'
+;; is set to `command-completion-default-include-p'.
+(defun tramp-dired-buffer-command-completion-p (_symbol buffer)
+  "A predicate for Tramp interactive commands.
+They are completed by `M-x TAB' only in Dired buffers."
+  (declare (tramp-suppress-trace t))
+  (with-current-buffer buffer
+    (tramp-dired-buffer-p)))
+
+;; FIXME: See FIXME above about renaming this before Emacs 31.1.
+
+;;;###autoload
+(defun tramp-dired-find-file-with-sudo ()
+  "Visit the file or directory named on this line as the superuser.
+
+By default this is done using the \"sudo\" Tramp method.
+YOu can customize `tramp-file-name-with-method' to change this.
+
+Interactively, with a prefix argument, prompt for a different method."
+  ;; (declare (completion tramp-dired-buffer-command-completion-p))
+  (interactive)
+  (with-tramp-file-name-with-method
+    (find-file (tramp-file-name-with-sudo (dired-get-file-for-visit)))))
+
+;; `tramp-dired-buffer-command-completion-p' is not autoloaded, and this
+;; setting isn't either.
+(function-put
+ #'tramp-dired-find-file-with-sudo 'completion-predicate
+ #'tramp-dired-buffer-command-completion-p)
 
 ;;; Recompile on ELPA
 
-;; This function takes action since Emacs 28.1, when
-;; `read-extended-command-predicate' is set to
-;; `command-completion-default-include-p'.
+;; This function takes action, when `read-extended-command-predicate'
+;; is set to `command-completion-default-include-p'.
 ;;;###tramp-autoload
 (defun tramp-recompile-elpa-command-completion-p (_symbol _buffer)
   "A predicate for `tramp-recompile-elpa'.
@@ -645,7 +768,7 @@ Tramp is an installed ELPA package."
 (defun tramp-recompile-elpa ()
   "Recompile the installed Tramp ELPA package.
 This is needed if there are compatibility problems."
-  ;; (declare (completion tramp-recompile-elpa-command-completion-p))
+  (declare (completion tramp-recompile-elpa-command-completion-p))
   (interactive)
   ;; We expect just one Tramp package is installed.
   (when-let*
@@ -665,12 +788,6 @@ This is needed if there are compatibility problems."
 	 "--eval" (format "(byte-recompile-directory %S 0 t)" dir))
 	(message "Package `tramp' recompiled.")))))
 
-;; Starting with Emacs 28.1, this can be replaced by the "(declare ...)" form.
-;;;###tramp-autoload
-(function-put
- #'tramp-recompile-elpa 'completion-predicate
- #'tramp-recompile-elpa-command-completion-p)
-
 ;; Tramp version is useful in a number of situations.
 
 ;;;###tramp-autoload
@@ -831,7 +948,7 @@ buffer in your bug report.
   (insert "\nload-path shadows:\n==================\n")
   (ignore-errors
     (mapc
-     (lambda (x) (when (tramp-compat-string-search "tramp" x) (insert x "\n")))
+     (lambda (x) (when (string-search "tramp" x) (insert x "\n")))
      (split-string (list-load-path-shadows t) "\n")))
 
   ;; Append buffers only when we are in message mode.
