@@ -1,6 +1,6 @@
 ;;; tramp-integration.el --- Tramp integration into other packages  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2019-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2019-2025 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -59,8 +59,9 @@
 
 ;;; Fontification of `read-file-name':
 
-(defvar tramp-rfn-eshadow-overlay)
-(make-variable-buffer-local 'tramp-rfn-eshadow-overlay)
+;; An overlay covering the shadowed part of the filename (local to the
+;; minibuffer).
+(defvar-local tramp-rfn-eshadow-overlay nil)
 
 (defun tramp-rfn-eshadow-setup-minibuffer ()
   "Set up a minibuffer for `file-name-shadow-mode'.
@@ -104,7 +105,8 @@ been set up by `rfn-eshadow-setup-minibuffer'."
 		   (minibuffer-prompt-end)))
 	  ;; We do not want to send any remote command.
 	  (non-essential t))
-      (when (tramp-tramp-file-p (buffer-substring end (point-max)))
+      (when (and (tramp-tramp-file-p (buffer-substring end (point-max)))
+		 (not (file-name-quoted-p (buffer-substring end (point-max)))))
 	(save-excursion
 	  (save-restriction
 	    (narrow-to-region
@@ -273,6 +275,33 @@ NAME must be equal to `tramp-current-connection'."
 		  (delete (info-lookup->mode-cache 'symbol ',mode)
 			  (info-lookup->topic-cache 'symbol))))))))
 
+;;; Integration of new `:link' type in `defcustom':
+
+(define-widget 'tramp-info-link 'link
+  "A link to the Tramp info file."
+  :action 'tramp-widget-info-link-action)
+
+(defun tramp-widget-info-link-action (widget &optional _event)
+  "Open the info node specified by WIDGET.
+It's value must be a Tramp user option, indexed in the Tramp manual via
+`@vindex'."
+  (let* ((topic (widget-value widget))
+	 (pattern
+	  (rx "\n*" (1+ " ") (0+ nonl)
+	      (literal (if (stringp topic) topic (symbol-name topic)))
+	      (0+ nonl) ":" (1+ (any "\t "))
+	      (group (0+ nonl))
+	      "." (0+ (any "\t\n ")) "(line" (1+ " ")
+	      (group (1+ digit))
+	      ")")))
+    (info "(tramp) Variable Index")
+    (goto-char (point-min))
+    (when (re-search-forward pattern nil t)
+      (let ((nodename (concat "(tramp) " (match-string-no-properties 1)))
+	    (line (string-to-number (match-string 2))))
+	(info nodename)
+	(forward-line (- line 2))))))
+
 ;;; Integration of shortdoc.el:
 
 (tramp--with-startup
