@@ -1,6 +1,6 @@
 ;;; tramp-smb.el --- Tramp access functions for SMB servers  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2002-2024 Free Software Foundation, Inc.
+;; Copyright (C) 2002-2025 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -114,6 +114,7 @@ this variable \"client min protocol=NT1\"."
        "Read from server failed, maybe it closed the connection"
        "Call timed out: server did not respond"
        (: (+ (not blank)) ": command not found")
+       (: (+ (not blank)) " does not exist")
        "Server doesn't support UNIX CIFS calls"
        (| ;; Samba.
 	"ERRDOS"
@@ -161,6 +162,7 @@ this variable \"client min protocol=NT1\"."
 	"NT_STATUS_PASSWORD_MUST_CHANGE"
 	"NT_STATUS_RESOURCE_NAME_NOT_FOUND"
 	"NT_STATUS_REVISION_MISMATCH"
+	"NT_STATUS_RPC_SS_CONTEXT_MISMATCH"
 	"NT_STATUS_SHARING_VIOLATION"
 	"NT_STATUS_TRUSTED_RELATIONSHIP_FAILURE"
 	"NT_STATUS_UNSUCCESSFUL"
@@ -315,7 +317,7 @@ Operations not mentioned here will be handled by the default Emacs primitives.")
 ;; Options for remote processes via winexe.
 (defcustom tramp-smb-winexe-program "winexe"
   "Name of winexe client to run.
-If it isn't found in the local $PATH, the absolute path of winexe
+If it isn't found in the local $PATH, the absolute path of \"winexe\"
 shall be given.  This is needed for remote processes."
   :group 'tramp
   :version "24.3"
@@ -428,8 +430,7 @@ arguments to pass to the OPERATION."
 	  (t2 (tramp-tramp-file-p newname))
 	  target)
       (with-parsed-tramp-file-name (if t1 dirname newname) nil
-	;; `copy-directory-create-symlink' exists since Emacs 28.1.
-	(if (and (bound-and-true-p copy-directory-create-symlink)
+	(if (and copy-directory-create-symlink
 		 (setq target (file-symlink-p dirname))
 		 (tramp-equal-remote dirname newname))
 	    (make-symbolic-link
@@ -482,18 +483,19 @@ arguments to pass to the OPERATION."
 
 		(let* ((share (tramp-smb-get-share v))
 		       (localname (file-name-as-directory
-				   (tramp-compat-string-replace
+				   (string-replace
 				    "\\" "/" (tramp-smb-get-localname v))))
 		       (tmpdir    (tramp-compat-make-temp-name))
 		       (args      (list (concat "//" host "/" share) "-E"))
 		       (options   tramp-smb-options))
 
-		  (if (tramp-string-empty-or-nil-p user)
-		      (setq args (append args (list "-N")))
-		    (setq args (append args (list "-U" user))))
+		  (setq args
+			(append args
+			 (if (tramp-string-empty-or-nil-p user)
+			     (list "-N")
+			   (list "-U" (if domain (concat domain "/" user) user)))
+			 (when port (list "-p" port))))
 
-		  (when domain (setq args (append args (list "-W" domain))))
-		  (when port   (setq args (append args (list "-p" port))))
 		  (when tramp-smb-conf
 		    (setq args (append args (list "-s" tramp-smb-conf))))
 		  (while options
@@ -527,13 +529,13 @@ arguments to pass to the OPERATION."
 
 		  (unwind-protect
 		      (with-tramp-saved-connection-properties
-			  v '("process-name" "process-buffer")
+			  v '(" process-name" " process-buffer")
 			(with-temp-buffer
 			  ;; Set the transfer process properties.
 			  (tramp-set-connection-property
-			   v "process-name" (buffer-name (current-buffer)))
+			   v " process-name" (buffer-name (current-buffer)))
 			  (tramp-set-connection-property
-			   v "process-buffer" (current-buffer))
+			   v " process-buffer" (current-buffer))
 
 			  (when t1
 			    ;; The smbclient tar command creates
@@ -569,7 +571,7 @@ arguments to pass to the OPERATION."
 
 		;; Handle KEEP-DATE argument.
 		(when keep-date
-		  (tramp-compat-set-file-times
+		  (set-file-times
 		   newname
 		   (file-attribute-modification-time (file-attributes dirname))
 		   (unless ok-if-already-exists 'nofollow)))
@@ -597,66 +599,63 @@ KEEP-DATE has no effect in case NEWNAME resides on an SMB server.
 PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
   (setq filename (expand-file-name filename)
 	newname (expand-file-name newname))
-  (with-tramp-progress-reporter
-      (tramp-dissect-file-name
-       (if (tramp-tramp-file-p filename) filename newname))
-      0 (format "Copying %s to %s" filename newname)
 
-    (if (file-directory-p filename)
-	(copy-directory filename newname keep-date 'parents 'copy-contents)
+  (with-parsed-tramp-file-name
+      (if (tramp-tramp-file-p filename) filename newname) nil
+    (with-tramp-progress-reporter
+	v 0 (format "Copying %s to %s" filename newname)
 
-      (unless (file-exists-p filename)
-	(tramp-error
-	 (tramp-dissect-file-name
-	  (if (tramp-tramp-file-p filename) filename newname))
-	 'file-missing filename))
+      (if (file-directory-p filename)
+	  (copy-directory filename newname keep-date 'parents 'copy-contents)
 
-      ;; `file-local-copy' returns a file name also for a local file
-      ;; with `jka-compr-handler', so we cannot trust its result as
-      ;; indication for a remote file name.
-      (if-let* ((tmpfile
-		 (and (tramp-tramp-file-p filename) (file-local-copy filename))))
-	  ;; Remote filename.
-	  (condition-case err
-	      (rename-file tmpfile newname ok-if-already-exists)
-	    ((error quit)
-	     (delete-file tmpfile)
-	     (signal (car err) (cdr err))))
+	(tramp-barf-if-file-missing v filename
+	  ;; `file-local-copy' returns a file name also for a local
+	  ;; file with `jka-compr-handler', so we cannot trust its
+	  ;; result as indication for a remote file name.
+	  (if-let* ((tmpfile
+		     (and (tramp-tramp-file-p filename)
+			  (file-local-copy filename))))
+	      ;; Remote filename.
+	      (condition-case err
+		  (rename-file tmpfile newname ok-if-already-exists)
+		((error quit)
+		 (delete-file tmpfile)
+		 (signal (car err) (cdr err))))
 
-	;; Remote newname.
-	(when (and (file-directory-p newname)
-		   (directory-name-p newname))
-	  (setq newname
-		(expand-file-name (file-name-nondirectory filename) newname)))
+	    ;; Remote newname.
+	    (when (and (file-directory-p newname)
+		       (directory-name-p newname))
+	      (setq newname
+		    (expand-file-name
+		     (file-name-nondirectory filename) newname)))
 
-	(with-parsed-tramp-file-name newname nil
-	  (when (and (not ok-if-already-exists) (file-exists-p newname))
-	    (tramp-error v 'file-already-exists newname))
-	  (when (and (file-directory-p newname)
-		     (not (directory-name-p newname)))
-	    (tramp-error v 'file-error "File is a directory %s" newname))
+	    (when (and (not ok-if-already-exists) (file-exists-p newname))
+	      (tramp-error v 'file-already-exists newname))
+	    (when (and (file-directory-p newname)
+		       (not (directory-name-p newname)))
+	      (tramp-error v 'file-error "File is a directory %s" newname))
 
-	  (unless (tramp-smb-get-share v)
-	    (tramp-error
-	     v 'file-error "Target `%s' must contain a share name" newname))
-	  (unless (tramp-smb-send-command
-		   v (format "put %s %s"
-			     (tramp-smb-shell-quote-argument filename)
-			     (tramp-smb-shell-quote-localname v)))
-	    (tramp-error
-	     v 'file-error "Cannot copy `%s' to `%s'" filename newname))
+	    (unless (tramp-smb-get-share v)
+	      (tramp-error
+	       v 'file-error "Target `%s' must contain a share name" newname))
+	    (unless (tramp-smb-send-command
+		     v (format "put %s %s"
+			       (tramp-smb-shell-quote-argument filename)
+			       (tramp-smb-shell-quote-localname v)))
+	      (tramp-error
+	       v 'file-error "Cannot copy `%s' to `%s'" filename newname))
 
-	  ;; When newname did exist, we have wrong cached values.
-	  (when (tramp-tramp-file-p newname)
-	    (with-parsed-tramp-file-name newname v2
-	      (tramp-flush-file-properties v2 v2-localname))))))
+	    ;; When newname did exist, we have wrong cached values.
+	    (when (tramp-tramp-file-p newname)
+	      (with-parsed-tramp-file-name newname v2
+		(tramp-flush-file-properties v2 v2-localname))))))
 
-    ;; KEEP-DATE handling.
-    (when keep-date
-      (tramp-compat-set-file-times
-       newname
-       (file-attribute-modification-time (file-attributes filename))
-       (unless ok-if-already-exists 'nofollow)))))
+      ;; KEEP-DATE handling.
+      (when keep-date
+	(set-file-times
+	 newname
+	 (file-attribute-modification-time (file-attributes filename))
+	 (unless ok-if-already-exists 'nofollow))))))
 
 (defun tramp-smb-handle-delete-directory (directory &optional recursive trash)
   "Like `delete-directory' for Tramp files."
@@ -713,7 +712,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
     (setq name "."))
   ;; Unless NAME is absolute, concat DIR and NAME.
   (unless (file-name-absolute-p name)
-    (setq name (tramp-compat-file-name-concat dir name)))
+    (setq name (file-name-concat dir name)))
   ;; If NAME is not a Tramp file, run the real handler.
   (if (not (tramp-tramp-file-p name))
       (tramp-run-real-handler #'expand-file-name (list name))
@@ -738,7 +737,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	(tramp-error v 'file-error "Cannot expand tilde in file `%s'" name))
       (unless (tramp-run-real-handler #'file-name-absolute-p (list localname))
 	(setq localname (concat "/" localname)))
-     ;; Do not keep "/..".
+      ;; Do not keep "/..".
       (when (string-match-p (rx bos "/" (** 1 2 ".") eos) localname)
 	(setq localname "/"))
       ;; Do normal `expand-file-name' (this does "/./" and "/../"),
@@ -777,17 +776,18 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
       (with-tramp-file-property v localname "file-acl"
 	(when (tramp-smb-remote-acl-p v)
 	  (let* ((share     (tramp-smb-get-share v))
-		 (localname (tramp-compat-string-replace
+		 (localname (string-replace
 			     "\\" "/" (tramp-smb-get-localname v)))
 		 (args      (list (concat "//" host "/" share) "-E"))
 		 (options   tramp-smb-options))
 
-	    (if (tramp-string-empty-or-nil-p user)
-		(setq args (append args (list "-N")))
-	      (setq args (append args (list "-U" user))))
+	    (setq args
+		  (append args
+		   (if (tramp-string-empty-or-nil-p user)
+		       (list "-N")
+		     (list "-U" (if domain (concat domain "/" user) user)))
+		   (when port (list "-p" port))))
 
-	    (when domain (setq args (append args (list "-W" domain))))
-	    (when port   (setq args (append args (list "-p" port))))
 	    (when tramp-smb-conf
 	      (setq args (append args (list "-s" tramp-smb-conf))))
 	    (while options
@@ -800,13 +800,13 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 				(concat "2>" (tramp-get-remote-null-device v)))))
 
 	    (with-tramp-saved-connection-properties
-		v '("process-name" "process-buffer")
+		v '(" process-name" " process-buffer")
 	      (with-temp-buffer
 		;; Set the transfer process properties.
 		(tramp-set-connection-property
-		 v "process-name" (buffer-name (current-buffer)))
+		 v " process-name" (buffer-name (current-buffer)))
 		(tramp-set-connection-property
-		 v "process-buffer" (current-buffer))
+		 v " process-buffer" (current-buffer))
 
 		;; Use an asynchronous process.  By this, password
 		;; can be handled.
@@ -842,8 +842,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 
 	    ;; Check result.
 	    (when entry
-	      (list (and (tramp-compat-string-search "d" (nth 1 entry))
-			 t)              ;0 file type
+	      (list (and (string-search "d" (nth 1 entry)) t) ;0 file type
 		    -1                   ;1 link count
 		    (cons
 		     tramp-unknown-id-string tramp-unknown-id-integer) ;2 uid
@@ -978,7 +977,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	   (mapcar
 	    (lambda (x)
 	      (list
-	       (if (tramp-compat-string-search "d" (nth 1 x))
+	       (if (string-search "d" (nth 1 x))
 		   (file-name-as-directory (nth 0 x))
 		 (nth 0 x))))
 	    (tramp-smb-get-file-entries directory))))))))
@@ -1017,7 +1016,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 (defun tramp-smb-handle-file-writable-p (filename)
   "Like `file-writable-p' for Tramp files."
   (if (file-exists-p filename)
-      (tramp-compat-string-search
+      (string-search
        "w" (or (file-attribute-modes (file-attributes filename)) ""))
     (let ((dir (file-name-directory filename)))
       (and (file-exists-p dir)
@@ -1080,14 +1079,14 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		(sort
 		 entries
 		 (lambda (x y)
-		   (if (tramp-compat-string-search "t" switches)
+		   (if (string-search "t" switches)
 		       ;; Sort by date.
 		       (time-less-p (nth 3 y) (nth 3 x))
 		     ;; Sort by name.
 		     (string-lessp (nth 0 x) (nth 0 y))))))
 
 	  ;; Handle "-F" switch.
-	  (when (tramp-compat-string-search "F" switches)
+	  (when (string-search "F" switches)
 	    (mapc
 	     (lambda (x)
 	       (unless (string-empty-p (car x))
@@ -1118,7 +1117,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 			   (expand-file-name
 			    (nth 0 x) (file-name-directory filename))
 			   'string)))))
-		 (when (tramp-compat-string-search "l" switches)
+		 (when (string-search "l" switches)
 		   (insert
 		    (format
 		     "%10s %3d %-8s %-8s %8s %s "
@@ -1147,7 +1146,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		   (put-text-property start (point) 'dired-filename t))
 
 		 ;; Insert symlink.
-		 (when (and (tramp-compat-string-search "l" switches)
+		 (when (and (string-search "l" switches)
 			    (stringp (file-attribute-type attr)))
 		   (insert " -> " (file-attribute-type attr))))
 
@@ -1249,12 +1248,13 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
       ;; Call it.
       (condition-case nil
 	  (with-tramp-saved-connection-properties
-	      v '("process-name" "process-buffer")
+	      v '(" process-name" " process-buffer")
 	    ;; Set the new process properties.
-	    (tramp-set-connection-property v "process-name" name1)
+	    (tramp-set-connection-property v " process-name" name1)
 	    (tramp-set-connection-property
-	     v "process-buffer"
+	     v " process-buffer"
 	     (or outbuf (generate-new-buffer tramp-temp-buffer-name)))
+	    (tramp-flush-connection-property v " process-exit-status")
 	    (with-current-buffer (tramp-get-connection-buffer v)
 	      ;; Preserve buffer contents.
 	      (narrow-to-region (point-max) (point-max))
@@ -1289,9 +1289,9 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
       ;; Cleanup.  We remove all file cache values for the connection,
       ;; because the remote process could have changed them.
       (when tmpinput (delete-file tmpinput))
-      ;; FIXME: Does connection-property "process-buffer" still exist?
+      ;; FIXME: Does connection-property " process-buffer" still exist?
       (unless outbuf
-	(kill-buffer (tramp-get-connection-property v "process-buffer")))
+	(kill-buffer (tramp-get-connection-property v " process-buffer")))
       (when process-file-side-effects
 	(tramp-flush-directory-properties v "/"))
 
@@ -1308,46 +1308,45 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 
   (with-parsed-tramp-file-name
       (if (tramp-tramp-file-p filename) filename newname) nil
-    (unless (file-exists-p filename)
-      (tramp-error v 'file-missing filename))
-    (when (and (not ok-if-already-exists) (file-exists-p newname))
-      (tramp-error v 'file-already-exists newname))
-    (when (and (file-directory-p newname)
-	       (not (directory-name-p newname)))
-      (tramp-error v 'file-error "File is a directory %s" newname))
+    (tramp-barf-if-file-missing v filename
+      (when (and (not ok-if-already-exists) (file-exists-p newname))
+	(tramp-error v 'file-already-exists newname))
+      (when (and (file-directory-p newname)
+		 (not (directory-name-p newname)))
+	(tramp-error v 'file-error "File is a directory %s" newname))
 
-    (with-tramp-progress-reporter
-	v 0 (format "Renaming %s to %s" filename newname)
+      (with-tramp-progress-reporter
+	  v 0 (format "Renaming %s to %s" filename newname)
 
-      (if (and (not (file-exists-p newname))
-	       (tramp-equal-remote filename newname)
-	       (string-equal
-		(tramp-smb-get-share (tramp-dissect-file-name filename))
-		(tramp-smb-get-share (tramp-dissect-file-name newname))))
-	  ;; We can rename directly.
-	  (with-parsed-tramp-file-name filename v1
-	    (with-parsed-tramp-file-name newname v2
+	(if (and (not (file-exists-p newname))
+		 (tramp-equal-remote filename newname)
+		 (string-equal
+		  (tramp-smb-get-share (tramp-dissect-file-name filename))
+		  (tramp-smb-get-share (tramp-dissect-file-name newname))))
+	    ;; We can rename directly.
+	    (with-parsed-tramp-file-name filename v1
+	      (with-parsed-tramp-file-name newname v2
 
-	      ;; We must also flush the cache of the directory, because
-	      ;; `file-attributes' reads the values from there.
-	      (tramp-flush-file-properties v1 v1-localname)
-	      (tramp-flush-file-properties v2 v2-localname)
-	      (unless (tramp-smb-get-share v2)
-		(tramp-error
-		 v2 'file-error
-		 "Target `%s' must contain a share name" newname))
-	      (unless (tramp-smb-send-command
-		       v2 (format "rename %s %s"
-				  (tramp-smb-shell-quote-localname v1)
-				  (tramp-smb-shell-quote-localname v2)))
-		(tramp-error v2 'file-error "Cannot rename `%s'" filename))))
+		;; We must also flush the cache of the directory, because
+		;; `file-attributes' reads the values from there.
+		(tramp-flush-file-properties v1 v1-localname)
+		(tramp-flush-file-properties v2 v2-localname)
+		(unless (tramp-smb-get-share v2)
+		  (tramp-error
+		   v2 'file-error
+		   "Target `%s' must contain a share name" newname))
+		(unless (tramp-smb-send-command
+			 v2 (format "rename %s %s"
+				    (tramp-smb-shell-quote-localname v1)
+				    (tramp-smb-shell-quote-localname v2)))
+		  (tramp-error v2 'file-error "Cannot rename `%s'" filename))))
 
-	;; We must rename via copy.
-	(copy-file
-	 filename newname ok-if-already-exists 'keep-time 'preserve-uid-gid)
-	(if (file-directory-p filename)
-	    (delete-directory filename 'recursive)
-	  (delete-file filename))))))
+	  ;; We must rename via copy.
+	  (copy-file
+	   filename newname ok-if-already-exists 'keep-time 'preserve-uid-gid)
+	  (if (file-directory-p filename)
+	      (delete-directory filename 'recursive)
+	    (delete-file filename)))))))
 
 (defun tramp-smb-action-set-acl (proc vec)
   "Set ACL data."
@@ -1366,19 +1365,18 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 
       (when (and (stringp acl-string) (tramp-smb-remote-acl-p v))
 	(let* ((share     (tramp-smb-get-share v))
-	       (localname (tramp-compat-string-replace
-			   "\\" "/" (tramp-smb-get-localname v)))
+	       (localname (string-replace "\\" "/" (tramp-smb-get-localname v)))
 	       (args      (list (concat "//" host "/" share) "-E" "-S"
-				(tramp-compat-string-replace
-				 "\n" "," acl-string)))
+				(string-replace "\n" "," acl-string)))
 	       (options   tramp-smb-options))
 
-	  (if (tramp-string-empty-or-nil-p user)
-	      (setq args (append args (list "-N")))
-	    (setq args (append args (list "-U" user))))
+	  (setq args
+		(append args
+		 (if (tramp-string-empty-or-nil-p user)
+		     (list "-N")
+		   (list "-U" (if domain (concat domain "/" user) user)))
+		 (when port (list "-p" port))))
 
-	  (when domain (setq args (append args (list "-W" domain))))
-	  (when port   (setq args (append args (list "-p" port))))
 	  (when tramp-smb-conf
 	    (setq args (append args (list "-s" tramp-smb-conf))))
 	  (while options
@@ -1392,13 +1390,13 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 			      "||" "echo" "tramp_exit_status" "1")))
 
 	  (with-tramp-saved-connection-properties
-	      v '("process-name" "process-buffer")
+	      v '(" process-name" " process-buffer")
 	    (with-temp-buffer
 	      ;; Set the transfer process properties.
 	      (tramp-set-connection-property
-	       v "process-name" (buffer-name (current-buffer)))
+	       v " process-name" (buffer-name (current-buffer)))
 	      (tramp-set-connection-property
-	       v "process-buffer" (current-buffer))
+	       v " process-buffer" (current-buffer))
 
 	      ;; Use an asynchronous process.  By this, password
 	      ;; can be handled.
@@ -1454,7 +1452,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 	   p)
       (unwind-protect
 	  (with-tramp-saved-connection-properties
-	      v '("process-name" "process-buffer")
+	      v '(" process-name" " process-buffer")
 	    (save-excursion
 	      (save-restriction
 		(while (get-process name1)
@@ -1462,8 +1460,8 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		  (setq i (1+ i)
 			name1 (format "%s<%d>" name i)))
 		;; Set the new process properties.
-		(tramp-set-connection-property v "process-name" name1)
-		(tramp-set-connection-property v "process-buffer" buffer)
+		(tramp-set-connection-property v " process-name" name1)
+		(tramp-set-connection-property v " process-buffer" buffer)
 		;; Activate narrowing in order to save BUFFER contents.
 		(with-current-buffer (tramp-get-connection-buffer v)
 		  (let ((buffer-undo-list t))
@@ -1480,16 +1478,14 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		    (tramp-send-string v command)))
 		(setq p (tramp-get-connection-process v))
 		(when program
-		  (process-put p 'remote-command (cons program args))
-		  (tramp-set-connection-property
-		   p "remote-command" (cons program args)))
+		  (process-put p 'remote-command (cons program args)))
 		;; Return value.
 		p)))
 
 	;; Save exit.
 	;; FIXME: Does `tramp-get-connection-buffer' return the proper value?
 	(with-current-buffer (tramp-get-connection-buffer v)
-	  (if (tramp-compat-string-search tramp-temp-buffer-name (buffer-name))
+	  (if (string-search tramp-temp-buffer-name (buffer-name))
 	      (progn
 		(set-process-buffer (tramp-get-connection-process v) nil)
 		(kill-buffer (current-buffer)))
@@ -1762,12 +1758,10 @@ are listed.  Result is the list (LOCALNAME MODE SIZE MTIME)."
 	     mode (or (match-string 1 line) "")
 	     mode (format
 		    "%s%s"
-		    (if (tramp-compat-string-search "D" mode) "d" "-")
+		    (if (string-search "D" mode) "d" "-")
 		    (mapconcat
 		     (lambda (_x) "") "    "
-		     (format
-		      "r%sx"
-		      (if (tramp-compat-string-search "R" mode) "-" "w"))))
+		     (format "r%sx" (if (string-search "R" mode) "-" "w"))))
 	     line (substring line 0 -6))
 	  (cl-return))
 
@@ -1917,16 +1911,19 @@ If ARGUMENT is non-nil, use it as argument for
 	     (share    (setq args (list (concat "//" host "/" share))))
 	     (t        (setq args (list "-g" "-L" host ))))
 
-	    (if (tramp-string-empty-or-nil-p user)
-		(setq args (append args (list "-N")))
-	      (setq args (append args (list "-U" user))))
+	    (setq args
+		  (append args
+		   (if (tramp-string-empty-or-nil-p user)
+		       (list "-N")
+		     (list "-U" (if domain (concat domain "/" user) user)))
+		   (when port (list "-p" port))))
 
-	    (when domain (setq args (append args (list "-W" domain))))
-	    (when port   (setq args (append args (list "-p" port))))
 	    (when tramp-smb-conf
 	      (setq args (append args (list "-s" tramp-smb-conf))))
 	    (dolist (option options)
 	      (setq args (append args (list "--option" option))))
+	    ;; For debugging.
+	    (setq args (append args (list "-d" "1")))
 	    (when argument
 	      (setq args (append args (list argument))))
 
@@ -2037,6 +2034,8 @@ Removes smb prompt.  Returns nil if an error message has appeared."
   (when (tramp-file-name-port vec)
     (tramp-error vec 'file-error "Port not supported for remote processes"))
 
+  ;; In case of "NT_STATUS_RPC_SS_CONTEXT_MISMATCH", the remote server
+  ;; is a Samba server.  winexe cannot install the respective service there.
   (tramp-smb-maybe-open-connection
    vec
    (format
@@ -2048,12 +2047,14 @@ Removes smb prompt.  Returns nil if an error message has appeared."
   ;; Suppress "^M".  Shouldn't we specify utf8?
   (set-process-coding-system (tramp-get-connection-process vec) 'raw-text-dos)
 
-  ;; Set width to 128.  This avoids mixing prompt and long error messages.
+  ;; Set width to 128 ($bufsize.Width) or 102 ($winsize.Width),
+  ;; respectively.  $winsize.Width cannot be larger.  This avoids
+  ;; mixing prompt and long error messages.
   (tramp-smb-send-command vec "$rawui = (Get-Host).UI.RawUI")
   (tramp-smb-send-command vec "$bufsize = $rawui.BufferSize")
   (tramp-smb-send-command vec "$winsize = $rawui.WindowSize")
   (tramp-smb-send-command vec "$bufsize.Width = 128")
-  (tramp-smb-send-command vec "$winsize.Width = 128")
+  (tramp-smb-send-command vec "$winsize.Width = 102")
   (tramp-smb-send-command vec "$rawui.BufferSize = $bufsize")
   (tramp-smb-send-command vec "$rawui.WindowSize = $winsize"))
 
@@ -2080,5 +2081,7 @@ Removes smb prompt.  Returns nil if an error message has appeared."
 ;;   several places, especially in `tramp-smb-handle-insert-directory'.
 ;;
 ;; * Keep a separate connection process per share.
+;;
+;; * Keep a permanent connection process for `process-file'.
 
 ;;; tramp-smb.el ends here
